/* Quarticurve style class definition
 * Copyright (c) 2002 Red Hat, Inc.
 * Copyright (c) 2007 Kevin Kofler
 *
 * Authors: Bernhard Rosenkränzer <bero@redhat.com>
 *          Preston Brown <pbrown@redhat.com>
 * Ported to Qt 4 by Kevin Kofler <Kevin@tigcc.ticalc.org>
 *
 * Released under the GNU General Public License (GPL) version 2.
 */
#ifndef _QUARTICURVE_H_
#define _QUARTICURVE_H_ 1

#ifdef QT_PLUGIN
#  define Q_EXPORT_STYLE_QUARTICURVE
#else
#  define Q_EXPORT_STYLE_QUARTICURVE Q_EXPORT
#endif // QT_PLUGIN

#include <QCleanlooksStyle>
#include <qcommonstyle.h>
#include <qcolor.h>
#include <qpalette.h>
#include <q3intcache.h>
//Added by qt3to4:
#include <QPixmap>
#include <QEvent>

class Q_EXPORT_STYLE_QUARTICURVE QuarticurveStyle : public QCleanlooksStyle
{
	Q_OBJECT

public:
	QuarticurveStyle();
	virtual ~QuarticurveStyle();

	virtual void polish(QWidget *widget);
	virtual void unpolish(QWidget *widget);

	virtual void drawPrimitive(PrimitiveElement, const QStyleOption *, QPainter *,
	                           const QWidget * = 0) const;

	virtual void drawControl(ControlElement, const QStyleOption *, QPainter *,
	                         const QWidget * = 0) const;

/*	void drawControlMask(ControlElement, QPainter *, const QWidget *,
	                     const QRect &,
	                     const QStyleOption & = QStyleOption::Default) const;*/

	virtual QRect subElementRect(SubElement, const QStyleOption *,
	                             const QWidget * = 0) const;

	virtual void drawComplexControl(ComplexControl,
	                                const QStyleOptionComplex *, QPainter *,
	                                const QWidget *) const;

	virtual QRect subControlRect(ComplexControl,
	                             const QStyleOptionComplex *, SubControl,
	                             const QWidget * = 0) const;

	virtual int pixelMetric(PixelMetric, const QStyleOption * = 0,
	                        const QWidget * = 0) const;

	virtual QSize sizeFromContents(ContentsType, const QStyleOption *,
	                               const QSize &, const QWidget * = 0)
	                              const;

	virtual int styleHint(StyleHint, const QStyleOption * = 0,
	                      const QWidget * = 0, QStyleHintReturn * = 0)
	                     const;

	virtual QPixmap standardPixmap(StandardPixmap standardPixmap,
	                               const QStyleOption *opt,
	                               const QWidget *widget) const;

protected:
        struct QuarticurveColorData {
	  QRgb buttonColor;
	  QRgb spotColor;

	  QColor shades[8];
	  QColor spots[3];

	  QPixmap *radioPix[8];
	  QPixmap *radioMask;

	  QPixmap *checkPix[6];
      
      QPixmap *checkMark[2];

	  ~QuarticurveColorData();
	  
	  bool isGroup (const QColorGroup &cg) {
	    return cg.button().rgb() == buttonColor && cg.highlight().rgb() == spotColor;
	  }
	};
  
	bool eventFilter(QObject *object, QEvent *event);
        void drawGradientBox(QPainter *p, QRect const &r,
			     const QColorGroup &cg,
			     const QuarticurveColorData *cdata,
			     bool horiz,
			     double shade1, double shade2) const;
        void drawGradient(QPainter *p, QRect const &r,
			  const QColorGroup &cg,
			  double shade1, double shade2,
			  bool horiz) const;

        void getShade (const QColorGroup &cg, int shadenr, QColor &res) const;
        void drawTextRect (QPainter *p, const QRect &r,
			   const QColorGroup &cg, QStyle::State flags,
			   const QBrush *fill = 0) const;
        void drawLightBevel(QPainter *p, const QRect &r,
			    const QColorGroup &cg,
			    QStyle::State flags,
			    const QBrush *fill = 0,
	                    bool dark = false) const;

        QuarticurveColorData *realizeData (const QColorGroup &cg) const;
        const QuarticurveColorData *lookupData (const QColorGroup &cg) const;
  
  
        Q3IntCache<QuarticurveColorData> m_dataCache;
        static const double shadeFactors[8];

        void drawItem ( QPainter * painter, const QRect & rectangle, int alignment, const QPalette & cg, bool enabled, const QPixmap * pixmap, const QString & text, QPalette::ColorRole textRole = QPalette::NoRole ) const;

protected slots:
	QIcon standardIconImplementation(StandardPixmap standardPixmap,
	                                 const QStyleOption *opt,
	                                 const QWidget *widget) const;
};
#endif
